/***********************************************************************************
************************************************************************************

Simple Alcohol Tester with PIC12F683
and Sparkfun Bargraph Breakout Board

Copyright (C)2012 Giovanni Bernardo
http://www.settorezero.com
date:  04/03/12

Sensor used: 			MQ-3
MCU:					Microchip PIC12F683
Clock:					8MHz Internal
Compiler:				HI-TECH C Compiler for PIC10/12/16 MCUs Version 9.83

You may NOT distribute or sell this code without explicit authorization

THE SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT
WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT
LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS FOR A
PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT SHALL
WE ARE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF
PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE
THEREOF), ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER
SIMILAR COSTS, WHETHER ASSERTED ON THE BASIS OF CONTRACT, TORT
(INCLUDING NEGLIGENCE), BREACH OF WARRANTY, OR OTHERWISE.

************************************************************************************
************************************************************************************/

#define _XTAL_FREQ 8000000 // Used for delay routines included in Hi-Tech C Compiler
#include <htc.h>

// configuration
// internal clock with GP4 and GP5 used as IOs, Master Clear not used: GP3 can be used as input (note: input only, no output!)
__CONFIG (FOSC_INTOSCIO & WDTE_OFF & PWRTE_OFF & MCLRE_OFF & CP_OFF & CPD_OFF & BOREN_OFF & IESO_OFF & FCMEN_OFF);

// Connection between PIC12F683 and Sparkfun Bargraph Breakout Board
#define DATA		GP2		// GP2 -> SIN
#define LATCH		GP1		// GP1 -> LAT
#define CLOCK		GP0		// GP0 -> CLK

// Bargraph modes
#define MODE_DOT	0		// bargraph dot mode
#define MODE_BAR	1		// bargraph bar mode

// global variables
unsigned char mode=MODE_BAR; // actual bargraph mode
unsigned int advalue=0; // value from A/D converter
unsigned char conversion; // conversion value between A/D value and led number

// function prototypes
void main(void);
void BarGraphSend(unsigned char value, char mo);

void main(void)
        {
		OSCCON=0x70; // Internal oscillator running @ 8MHz
		while(!HTS); // waiting internal oscillator stability
		TRISIO=0x38; // GP0-1-2 as output, GP4(AN3) and other unused to input
		ANSEL=0x38; // A/D Clock=Frc, AN3(GP4)=analog input
		ADCON0=0x8D; // A/D result right justified, Vref=Vdd, read from AN3(GP4)
		
		while(1)
			{
			__delay_us(20); // Chold delay
			GO=1; // start conversion
			while(GO); // waiting end conversion
			advalue=ADRESL+(ADRESH<<8);
			// conversion between ad value and led number (0 - 29)
			conversion=(advalue*30)>>10; // note: right shift by 10 = division by 1024
			BarGraphSend(conversion,mode); // serialize data for shift registers
			} // \while
			
		} // \main
	
void BarGraphSend(unsigned char value, char mo)
	{
	unsigned char shift;
	unsigned long bar,v;
	bar=1ul<<value; // dot in "value" position
	if (mo==MODE_BAR) // bar mode? turn on all leds until dot position - 1
		{
		unsigned char u;	
		for (u=0; u<value; u++)
			{
			bar |= 1ul<<u; // add other dots
			}
		}
	// data serialization
	for (shift=0; shift<32; shift++) // serialize 32 bits even if there are only 30 leds
		{
		v=bar&(0x80000000>>shift); // advalue from MSB to LSB
		v?DATA=1:DATA=0;
		CLOCK=1;
		CLOCK=0;
		}
	LATCH=1;
	LATCH=0;
	} // \BarGraphSend
	
